<?php
use Automattic\WooCommerce\Blocks\Integrations\IntegrationInterface;
use Automattic\WooCommerce\Blocks\Package;
use Automattic\WooCommerce\Blocks\Domain\Services\CheckoutFields;
	/**
	* Class for integrating with WooCommerce Blocks
	*/
class AF_CF_Blocks_Integration implements IntegrationInterface {

	/**
	 * The name of the integration.
	 *
	 * @return stringregister_AF_CF_block_f
	 */
	public function get_name() {
		return 'af_c_f_block';
	}

	/**
	 * When called invokes any initialization/setup for the integration.
	 */
	public function initialize() {
		
		$this->register_main_integration();
		$this->register_AF_CF_block_frontend_scripts();
		$this->register_AF_CF_block_editor_scripts();
		add_action('wp_enqueue_scripts', array( $this, 'af_c_f_front_script' )  );
		add_action('wp_ajax_checkout_feild_manager_cookie_block_fee', array( $this, 'checkout_feild_manager_cookie_block_fee' ));
		add_action('wp_ajax_nopriv_checkout_feild_manager_cookie_block_fee', array( $this, 'checkout_feild_manager_cookie_block_fee' ));

	}
   
	public function checkout_feild_manager_cookie_block_fee() {

		$nonce = isset( $_POST['nonce'] ) && '' !== $_POST['nonce'] ? sanitize_text_field( wp_unslash( $_POST['nonce'] ) ) : 0;

		if ( empty( $_POST['nonce'] ) || ! wp_verify_nonce( $nonce, 'afcf-blocks-ajax-nonce' ) ) {
			die( esc_html__( 'Failed ajax security check!', 'af_checkout_fields' ) );
		}

		$mergedArray = array();
		foreach (WC()->session->get( 'checkout_block_fee_session') as $item) {
			$mergedArray[ $item['key'] ] = $item['value'];
		}

		
		$af_checkout_feild_manager_extension_data_arr= isset( $_POST['af_checkout_feild_manager_extension_data'] ) ? sanitize_meta('', $_POST['af_checkout_feild_manager_extension_data'], '' ) : sanitize_meta( $_POST['af_checkout_feild_manager_extension_data'] );

		foreach ($af_checkout_feild_manager_extension_data_arr as $item) {
			$mergedArray[ $item['key'] ] = $item['value'];
		}

	
		$resultArray = array();
		foreach ($mergedArray as $key => $value) {
		
			$resultArray[] = array(
				'key' => $key,
				'value' => $value,
			);
		}

		WC()->session->set( 'checkout_block_fee_session', $resultArray );

		$response = array(
			'checkout_block_fee_session'           => WC()->session->get( 'checkout_block_fee_session'), 
		);
		
		wp_send_json_success( $response );
	}

	public function af_c_f_front_script(){

		$user = wp_get_current_user();
		$user_roles = 'guest';
		if ( !empty( $user->roles[0] ) ) {
			$user_roles = $user->roles[0];
		
		}

		$af_checkout = new AF_C_F_Checkout();
		$checkout_fields = $af_checkout->get_checkout_fields();

		$field_data_array = array(); 
		
		foreach ($checkout_fields as $field_id) {

			$field_products   = get_post_meta( $field_id, 'af_c_f_field_products', true );
			$field_categories = get_post_meta( $field_id, 'af_c_f_field_categories', true );
			$field_tags       = get_post_meta( $field_id, 'af_c_f_field_tags', true );

			$match_product = '_feild_excluded';
			if ( empty( $field_products ) && empty( $field_categories ) && empty( $field_tags ) ) {
				$match_product = '_feild_included';
			}

			foreach ( wc()->cart->get_cart_contents() as $cart_item_key => $cart_item ) {

				$product_id = $cart_item['product_id'];

				if ( !empty( $field_products ) ) {
					if ( in_array($product_id, $field_products ) ) {
						$match_product = '_feild_included';
					}
				}

				if ( !empty( $field_categories ) ) {
					if ( has_term($field_categories, 'product_cat', $product_id ) ) {
						$match_product = '_feild_included';
					}
				}

				if ( !empty( $field_tags ) ) {
					if ( has_term($field_tags, 'product_tag', $product_id ) ) {
						$match_product = '_feild_included';
					}
				}
			}
	
			
			if (function_exists('has_shortcode')) {
				$all_pages = get_pages();
				foreach ($all_pages as $page) {
					$page_content = $page->post_content;
					if (has_shortcode($page_content, 'woocommerce_checkout')) {
						$match_product = '_feild_excluded';
					}
				}
			}
	
			
			
			$display_user_details = get_post_meta($field_id, 'af_c_f_display_user_details', true);
			
			$key = 'af_c_f_'.$field_id;
		
			$dep_fields='';
			
			
			$is_feild_dependable = get_post_meta($field_id, 'af_c_f_dependable', true);

			if ('yes'== $is_feild_dependable ) {
				$dep_fields = '#af_c_f_'.get_post_meta($field_id, 'af_c_f_dep_fields', true) . '_dependable';
			}


			$_type = get_post_meta($field_id, 'af_c_f_field_type', true);
			$_options = (array)get_post_meta($field_id, 'af_c_f_field_option', true);
			$_file_size = get_post_meta($field_id, 'af_c_f_field_file_size', true);
			$_file_type = get_post_meta($field_id, 'af_c_f_field_file_type', true);
			$_required = get_post_meta($field_id, 'af_c_f_field_required', true);
			$_read_only = get_post_meta($field_id, 'af_c_f_field_read_only', true);
			$_order_details = get_post_meta($field_id, 'af_c_f_field_order_details', true);
			$_width = get_post_meta($field_id, 'af_c_f_field_width', true);
			$_placeholder = get_post_meta($field_id, 'af_c_f_field_placeholder', true);
			$_description = get_post_meta($field_id, 'af_c_f_field_description', true);
			$_css = get_post_meta($field_id, 'af_c_f_field_css', true);
			$_heading_tag = get_post_meta($field_id, 'af_c_f_heading_tag', true);
			$_text = get_post_meta($field_id, 'af_c_f_field_text', true);
			$value = get_user_meta($user->ID, $key, true);
			$_location = get_post_meta( $field_id, 'af_c_f_checkout_position', true );
			$_options = empty($_options) ? array() : $_options;
			
			$custom_attributes = '';
		
			$main_class = 'half' == $_width ? 'half_width' : 'form-row-wide';
		

			$field_price         = get_post_meta( $field_id, 'af_c_f_field_price', true );
			$field_price_type    = get_post_meta( $field_id, 'af_c_f_field_price_type', true );
			$field_price_taxable = get_post_meta( $field_id, 'af_c_f_field_price_taxable', true );

			$price = 0;

			if ( !$af_checkout->is_field_has_options( $field_id ) ) {

				switch ( $field_price_type ) {
					case 'fixed':
						$price = floatval( $field_price );
						break;
					case 'percentage':
						$price = wc()->cart->get_subtotal() * ( floatval( $field_price ) / 100 );
						break;
				}
			}
		//	$_options=$af_checkout->get_options_of_field_display($_options,$field_price_type);
			$label       = get_the_title( $field_id );
			$field_price = !empty( $price ) ? '' . $price . '' : '';


			$field_data = array(
				'key' => $key,
				'dep_fields' => $dep_fields,
				'_type' => $_type,
				'_options' => $_options,
				'_file_size' => $_file_size,
				'_file_type' => $_file_type,
				'_required' => $_required,
				'_read_only' => $_read_only,
				'_order_details' => $_order_details,
				'_width' => $_width,
				'_placeholder' => $_placeholder,
				'_description' => $_description,
				'_css' => $_css,
				'_heading_tag' => $_heading_tag,
				'_text' => $_text,
				'value' => $value,
				'custom_attributes' => $custom_attributes,
				'main_class' => $main_class,
				'_location_checkout_block'=>$_location,
				'feild_title'=>$label,
				'feild_price'=>$field_price,
				'Is_feild_to_display'=>$match_product,
			);
		
		
			$field_data_array[] = $field_data;
		}
	
		$Short_code_check='_wc_sh_excluded';
		if (function_exists('has_shortcode')) {
			$all_pages = get_pages();
			foreach ($all_pages as $page) {
				$page_content = $page->post_content;
				if (has_shortcode($page_content, 'woocommerce_checkout')) {
					$Short_code_check = '_wc_sh_included';
				}
			}
		}



		if ('_wc_sh_included'==$Short_code_check) {

			$checkout_block_session_check=array();
		
		} else {

			$checkout_block_session_check=WC()->session->get( 'checkout_block_fee_session');
		
		}


	$var_bill_field_arr='';

	$var_ship_field_arr='';
	
	
	
		if ('_wc_sh_included'==$Short_code_check) {
			$var_bill_field_arr='not_found';

			$var_ship_field_arr='not_found';


		} else {
			if (!empty(get_option( 'af_checkout_billing_fields' ))) {
				$var_bill_field_arr=get_option( 'af_checkout_billing_fields' );
			} else {
				$var_bill_field_arr='not_found';
			}
	
			if (!empty(get_option( 'af_checkout_shipping_fields' ))) {
				$var_ship_field_arr=get_option( 'af_checkout_shipping_fields' );
			} else {
				$var_ship_field_arr='not_found';
			}
	   
		}

		
	


		wp_enqueue_script( 'afcf-blocks-frontj', plugins_url( '/src/js/af-c-f-block/block.js', __FILE__ ), '', '1.0', false  );
		wp_enqueue_script( 'jquery' );
		$afcf_block_data = array(
			'admin_url'                                    => admin_url( 'admin-ajax.php' ),
			'nonce'                                        =>wp_create_nonce( 'afcf-blocks-ajax-nonce' ),
			'checkout_feilds'                              =>$field_data_array,
			'google_recapta_site_key'                      =>get_option('af_c_f_site_key'),
			'af_checkout_feilds_bill'                      =>$var_bill_field_arr,
			'af_checkout_feilds_shiped'                    =>$var_ship_field_arr,
			'af_cf_crnt_usr_role'                           =>$user_roles,
			'af_cf_checkout_block_fee_session'             =>$checkout_block_session_check,
		);
		
		wp_localize_script( 'afcf-blocks-frontj', 'afcf_php_vars', $afcf_block_data );



    }

	/**
	 * Registers the main JS file required to add filters and Slot/Fills.
	 */
	public function register_main_integration() {
		$script_path = '/build/index.js';
		$script_url  = plugins_url( $script_path, __FILE__ );
		$script_asset_path = __DIR__ . '/build/index.asset.php';
		$script_asset      = file_exists( $script_asset_path )
			? require $script_asset_path
			: array(
				'dependencies' => array(),
				'version'      => $this->get_file_version( $script_path ),
			);


		wp_register_script(
			'af-c-f-blocks-integration',
			$script_url,
			$script_asset['dependencies'],
			$script_asset['version'],
			true
		);
		wp_set_script_translations(
			'af-c-f-blocks-integration',
			'af_c_f_block',
			__DIR__ . '/languages'
		);
	}

	/**
	 * Returns an array of script handles to enqueue in the frontend context.
	 *
	 * @return string[]
	 */
	public function get_script_handles() {
		return array( 'af-c-f-blocks-integration', 'af-c-f-block-frontend' );
	}

	/**
	 * Returns an array of script handles to enqueue in the editor context.
	 *
	 * @return string[]
	 */
	public function get_editor_script_handles() {
		return array( 'af-c-f-blocks-integration', 'af-c-f-block-editor' );
	}

	/**
	 * An array of key, value pairs of data made available to the block on the client side.
	 *
	 * @return array
	 */
	public function get_script_data() {
		$data = array(
			'af_c_f_block-active'    => true,
			'defaultLabelText' => __( 'default custom feild heading', 'af_c_f_block' ),
		);

		return $data;
	}

	

	public function register_AF_CF_block_editor_scripts() {
		$script_path       = '/build/af-c-f-block.js';
		$script_url        = plugins_url( $script_path, __FILE__ );
		$script_asset_path = __DIR__ . '/build/af-c-f-block.asset.php';
		$script_asset      = file_exists( $script_asset_path )
			? require $script_asset_path
			: array(
				'dependencies' => array(),
				'version'      => $this->get_file_version( $script_asset_path ),
			);

		wp_register_script(
			'af-c-f-block-editor',
			$script_url,
			$script_asset['dependencies'],
			$script_asset['version'],
			true
		);

		wp_set_script_translations(
			'af-c-f-block-editor',
			'af_c_f_block',
			__DIR__ . '/languages'
		);
	}

	public function register_AF_CF_block_frontend_scripts() {
		$script_path       = '/build/af-c-f-block-frontend.js';
		$script_url        = plugins_url( $script_path, __FILE__ );
		$script_asset_path = __DIR__ . '/build/af-c-f-block-frontend.asset.php';
		$script_asset      = file_exists( $script_asset_path )
			? require $script_asset_path
			: array(
				'dependencies' => array(),
				'version'      => $this->get_file_version( $script_asset_path ),
			);

		wp_register_script(
			'af-c-f-block-frontend',
			$script_url,
			$script_asset['dependencies'],
			$script_asset['version'],
			true
		);
		wp_set_script_translations(
			'af-c-f-block-frontend',
			'af_c_f_block',
			__DIR__ . '/languages'
		);
	}

	/**
	 * Get the file modified time as a cache buster if we're in dev mode.
	 *
	 * @param string $file Local path to the file.
	 * @return string The cache buster value to use for the given file.
	 */
	protected function get_file_version( $file ) {
		if ( defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG && file_exists( $file ) ) {
			return filemtime( $file );
		}
		return AF_CF_BLOCK_VERSION;
	}
}
